@@ -264,29 +264,43 @@ def segment_dat(dat, marker_def, ival, newsamples=None, timeaxis=-2):
     data = []
     classes = []
     class_names = sorted(marker_def.keys())
+    masks = []
     for t, m in dat.markers:
         for class_idx, classname in enumerate(class_names):
             if m in marker_def[classname]:
                 mask = (t+ival[0] <= dat.axes[timeaxis]) & (dat.axes[timeaxis] < t+ival[1])
-                d = dat.data.compress(mask, timeaxis)
-                d = np.expand_dims(d, axis=0)
-                if d.shape[timeaxis] != expected_samples:
+                # this one is quite expensive, we should move this stuff
+                # out of the loop, or eliminate the loop completely
+                # e.g. np.digitize for the marker to timeaxis mapping
+                mask = np.flatnonzero(mask)
+                if len(mask) != expected_samples:
                     # result is too short or too long, ignore it
                     continue
                 # check if the new cnt shares at least one timepoint
                 # with the new samples. attention: we don't only have to
                 # check the ival but also the marker if it is on the
                 # right side of the ival!
-                times = dat.axes[timeaxis].compress(mask)
+                times = dat.axes[timeaxis].take(mask)
                 if newsamples is not None:
                     if newsamples == 0:
                         continue
                     if (len(np.intersect1d(times, new_sample_times)) == 0 and
                         t < new_sample_times[0]):
                         continue
-                data.append(d)
+                masks.append(mask)
                 classes.append(class_idx)
-    data = np.concatenate(data, axis=0) if len(data) > 0 else np.array(data)
+    if len(masks) == 0:
+        data = np.array([])
+    else:
+        # np.take inserts a new dimension at `axis`...
+        data = dat.data.take(masks, axis=timeaxis)
+        # we want that new dimension at axis 0 so we have to swap it.
+        # before that we have to convert the netagive axis indices to
+        # their equivalent positive one, otherwise swapaxis will be one
+        # off.
+        if timeaxis < 0:
+            timeaxis = dat.data.ndim + timeaxis
+        data = data.swapaxes(0, timeaxis)
     axes = dat.axes[:]
     time = np.linspace(ival[0], ival[1], (ival[1] - ival[0]) / 1000 * dat.fs, endpoint=False)
     axes[timeaxis] = time
@@ -508,8 +522,7 @@ def append_epo(dat, dat2, classaxis=0, extra=None):
 
 
 def lfilter(dat, b, a, zi=None, timeaxis=-2):
-    """
-    Filter data using the filter defined by the filter coefficients.
+    """Filter data using the filter defined by the filter coefficients.
 
     This method mainly delegates the call to
     :func:`scipy.signal.lfilter`.
@@ -521,7 +534,7 @@ def lfilter(dat, b, a, zi=None, timeaxis=-2):
     b : 1-d array
         the numerator coefficient vector
     a : 1-d array
-        the denominator cefficient vector
+        the denominator coefficient vector
     zi : nd array, optional
         the initial conditions for the filter delay. If zi is ``None``
         or not given, initial rest is assumed.
@@ -535,8 +548,8 @@ def lfilter(dat, b, a, zi=None, timeaxis=-2):
 
     See Also
     --------
-    :func:`scipy.signal.lfilter`, :func:`scipy.signal.butter`,
-    :func:`scipy.signal.butterord`
+    :func:`filtfilt`, :func:`scipy.signal.lfilter`,
+    :func:`scipy.signal.butter`, :func:`scipy.signal.butterord`
 
     Examples
     --------
@@ -586,6 +599,60 @@ def lfilter(dat, b, a, zi=None, timeaxis=-2):
         return dat.copy(data=data), zo
 
 
+def filtfilt(dat, b, a, timeaxis=-2):
+    """A forward-backward filter.
+
+    Filter data twice, once forward and once backwards, using the filter
+    defined by the filter coefficients.
+
+    This method mainly delegates the call to
+    :func:`scipy.signal.filtfilt`.
+
+    Parameters
+    ----------
+    dat : Data
+        the data to be filtered
+    b : 1-d array
+        the numerator coefficient vector
+    a : 1-d array
+        the denominator coefficient vector
+    timeaxis : int, optional
+        the axes in ``data`` to filter along to
+
+    Returns
+    -------
+    dat : Data
+        the filtered output
+
+    See Also
+    --------
+    :func:`lfilter`
+
+    Examples
+    --------
+
+    Generate and use a Butterworth bandpass filter for complete
+    (off-line data):
+
+    >>> # the sampling frequency of our data in Hz
+    >>> dat.fs
+    100
+    >>> # calculate the nyquist frequency
+    >>> fn = dat.fs / 2
+    >>> # the desired low and high frequencies in Hz
+    >>> f_low, f_high = 2, 13
+    >>> # the order of the filter
+    >>> butter_ord = 4
+    >>> # calculate the filter coefficients
+    >>> b, a = signal.butter(butter_ord, [f_low / fn, f_high / fn], btype='band')
+    >>> filtered = filtfilt(dat, b, a)
+
+    """
+    # TODO: should we use padlen and padtype?
+    data = signal.filtfilt(b, a, dat.data, axis=timeaxis)
+    return dat.copy(data=data)
+
+
 def clear_markers(dat, timeaxis=-2):
     """Remove markers that are outside of the ``dat`` time interval.
 
@@ -951,27 +1018,70 @@ def subsample(dat, freq, timeaxis=-2):
     return dat.copy(data=data, axes=axes, fs=freq)
 
 
-def spectrum(cnt):
-    """Calculate the normalized spectrum of a continuous data object.
+def spectrum(dat, timeaxis=-2):
+    """Calculate the spectrum of a data object.
 
+    This method performs a fast fourier transform on the data along the
+    timeaxis and returns a new `Data` object which is transformed into
+    the frequency domain. The values are the amplitudes of of the
+    respective frequencies.
+
+    Parameters
+    ----------
+    dat : Data
+        Data object with `.fs` attribute
+    timeaxis : int, optional
+        axis to perform the fft along
 
     Returns
     -------
-    fourier : ndarray
-    freqs : ndarray
+    dat : Data
+        Data object with the timeaxis transformed into the frequency
+        domain. The values of the spectrum are the amplitudes of the
+        respective frequencies.
+
+    Examples
+    --------
+    >>> # dat can be continuous or epoched
+    >>> dat.axes
+    ['time', 'channel']
+    >>> spm = spectrum(dat)
+    >>> spm.axes
+    ['frequency', 'channel']
+
+    Raises
+    ------
+    AssertionError
+        if the `dat` paramter has no `.fs` attribute
 
     See Also
     --------
     spectrogram, stft
 
     """
-    fourier = np.array([sp.fftpack.rfft(cnt.data[:,i]) for i in range(cnt.data.shape[-1])])
-    fourier *= (2 / cnt.data.shape[-2])
-    freqs = sp.fftpack.rfftfreq(cnt.data.shape[-2], 1/cnt.fs)
-    return fourier, freqs
+    # oh look at that! a dumb idea just found a friend.
+    assert hasattr(dat, 'fs')
+    # number of samples of our data
+    length = dat.data.shape[timeaxis]
+    fourier = sp.fftpack.fft(dat.data, axis=timeaxis)
+    fourier = fourier.take(np.arange(length)[1:length/2], axis=timeaxis)
+    amps = 2 * fourier / length
+    amps = np.abs(amps)
+    freqs = sp.fftpack.fftfreq(length, 1/dat.fs)
+    freqs = freqs[1:length/2]
+    axes = dat.axes[:]
+    axes[timeaxis] = freqs
+    names = dat.names[:]
+    names[timeaxis] = 'frequency'
+    units = dat.units[:]
+    units[timeaxis] = 'dl'
+    # TODO: units in original unit or dimensionles?)
+    spm = dat.copy(data=amps, axes=axes, names=names, units=units)
+    delattr(spm, 'fs')
+    return spm
 
 
-def spectrogram(cnt):
+def spectrogram(cnt, timeaxis=-2):
     """Calculate the spectrogram of a continuous data object.
 
     See Also
@@ -979,14 +1089,15 @@ def spectrogram(cnt):
     spectrum, stft
 
     """
-    framesize = 1000 #ms
-    width = int(cnt.fs * (framesize / 1000))
-    specgram = np.array([stft(cnt.data[:,i], width) for i in range(cnt.data.shape[-1])])
-    freqs = sp.fftpack.rfftfreq(width, 1/cnt.fs)
+    framesize = 2000 #ms
+    width = int(cnt.fs * framesize / 1000)
+    specgram = stft(cnt.data, width, timeaxis=timeaxis)
+    freqs = sp.fftpack.fftfreq(width, 1/cnt.fs)[1:width/2]
+    print freqs
     return specgram, freqs
 
 
-def stft(x, width):
+def stft(x, width, timeaxis=-2):
     """Short time fourier transform of a real sequence.
 
     This method performs a discrete short time Fourier transform. It
@@ -1016,9 +1127,10 @@ def stft(x, width):
 
     """
     window = sp.hanning(width)
-    fourier = np.array([sp.fftpack.rfft(x[i:i+width] * window) for i in range(0, len(x)-width, width//2)])
-    fourier *= (2 / width)
-    return fourier
+    fourier = np.array([sp.fftpack.fft(x[i:i+width] * window)[1:width/2] for i in range(0, len(x)-width)])#, width//2)])
+    amps = 2 * fourier / width
+    amps = np.abs(amps)
+    return amps
 
 
 def calculate_csp(class1, class2):
@@ -1206,9 +1318,9 @@ def correct_for_baseline(dat, ival, timeaxis=-2):
     assert dat.axes[timeaxis][0] <= ival[0] <= ival[1]
     mask = (ival[0] <= dat.axes[timeaxis]) & (dat.axes[timeaxis] < ival[1])
     # take all values from the dat except the ones not fitting the mask
-    # and calculate the average along the sampling axis
-    averages = np.average(dat.data.compress(mask, timeaxis), axis=timeaxis)
-    data = dat.data - np.expand_dims(averages, timeaxis)
+    # and calculate the mean along the sampling axis
+    means = np.mean(dat.data.compress(mask, timeaxis), axis=timeaxis)
+    data = dat.data - np.expand_dims(means, timeaxis)
     return dat.copy(data=data)
 
 
@@ -1363,7 +1475,7 @@ def calculate_signed_r_square(dat, classaxis=0):
 
 
 def logarithm(dat):
-    """Computes the element wise natural logarithm of ``dat.data``
+    """Computes the element wise natural logarithm of ``dat.data``.
 
     Calling this method is equivalent to calling
 
@@ -1372,6 +1484,7 @@ def logarithm(dat):
     Parameters
     ----------
     dat : Data
+        a Data object
 
     Returns
     -------
@@ -1379,14 +1492,48 @@ def logarithm(dat):
         a copy of ``dat`` with the element wise natural logarithms of
         the values in ``.data``
 
+    See Also
+    --------
+    :func:`square`
+
     """
     data = np.log(dat.data)
     return dat.copy(data=data)
 
 
+def square(dat):
+    """Computes the element wise square of ``dat.data``.
+
+    Calling this method is equivalent to calling
+
+    >>> dat.copy(data=np.square(dat.data))
+
+    Parameters
+    ----------
+    dat : Data
+        a Data object
+
+
+    Returns
+    -------
+    dat : Data
+        a copy of ``dat`` with the element wise squares of the values in
+        ``.data``
+
+    See Also
+    --------
+    :func:`logarithm`
+
+    """
+    data = np.square(dat.data)
+    return dat.copy(data=data)
+
+
 def variance(dat, timeaxis=-2):
     """Compute the variance along the ``timeaxis`` of ``dat``.
 
+    This method reduces the dimensions of `dat.data` by one.
+
     Parameters
     ----------
     dat : Data
@@ -1408,8 +1555,11 @@ def variance(dat, timeaxis=-2):
 
     """
     data = np.var(dat.data, axis=timeaxis)
-    axes = dat.axes[:].pop(timeaxis)
-    names = dat.names[:].pop(timeaxis)
-    units = dat.units[:].pop(timeaxis)
+    axes = dat.axes[:]
+    axes.pop(timeaxis)
+    names = dat.names[:]
+    names.pop(timeaxis)
+    units = dat.units[:]
+    units.pop(timeaxis)
     return dat.copy(data=data, axes=axes, names=names, units=units)
 
